@@ -9,7 +9,7 @@ use 5.014;
 #use strict;
 #use warnings;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 our %TRUE_VALUES = (
                     'true' => 1,
@@ -112,23 +112,29 @@ sub parse {
 
     foreach my $desktop_file (@desktop_files) {
 
+        # Check the filename and skip it if it matches `skip_filename_re`
         if (defined $self->{skip_filename_re}) {
             substr($desktop_file, rindex($desktop_file, '/') + 1) =~ /$self->{skip_filename_re}/ && next;
         }
 
+        # Open and read the desktop file
         sysopen my $desktop_fh, $desktop_file, 0 or next;
         sysread $desktop_fh, (my $file), -s $desktop_file;
 
+        # Locate the "[Desktop Entry]" section
         if ((my $index = index($file, "]\n", index($file, "[Desktop Entry]") + 15)) != -1) {
             $file = substr($file, 0, $index);
         }
 
+        # Parse the entry data
         my %info = $file =~ /$self->{_file_keys_re}/g;
 
+        # Ignore the file when `NoDisplay` is true
         if (exists $info{NoDisplay}) {
             next if exists $TRUE_VALUES{$info{NoDisplay}};
         }
 
+        # Ignore the file when `Hidden` is true
         if (exists $info{Hidden}) {
             next if exists $TRUE_VALUES{$info{Hidden}};
         }
@@ -136,15 +142,7 @@ sub parse {
         # If no 'Name' entry is defined, create one with the name of the file
         $info{Name} //= substr($desktop_file, rindex($desktop_file, '/') + 1, -8);
 
-        (
-         my @categories =
-           grep { exists $self->{_categories}{$_} }
-           $self->{case_insensitive_cats}
-         ? (map { lc($_) =~ tr/_a-z0-9/_/cr } split(/;/, $info{Categories} // ''))
-         : (split(/;/, $info{Categories} // ''))
-        )
-          || (!$self->{keep_unknown_categories} and next);
-
+        # Handle `skip_entry`
         if (defined($self->{skip_entry}) and ref($self->{skip_entry}) eq 'ARRAY') {
             my $skip;
             foreach my $pair_ref (@{$self->{skip_entry}}) {
@@ -156,6 +154,7 @@ sub parse {
             $skip and next;
         }
 
+        # Make user-defined substitutions
         if (defined($self->{substitutions}) and ref($self->{substitutions}) eq 'ARRAY') {
             foreach my $pair_ref (@{$self->{substitutions}}) {
                 if (exists $info{$pair_ref->{key}}) {
@@ -169,25 +168,50 @@ sub parse {
             }
         }
 
+        # Parse categories
+        (
+         my @categories =
+           grep { exists $self->{_categories}{$_} }
+           $self->{case_insensitive_cats}
+         ? (map { lc($_) =~ tr/_a-z0-9/_/cr } split(/;/, $info{Categories} // ''))
+         : (split(/;/, $info{Categories} // ''))
+        )
+          || (!$self->{keep_unknown_categories} and next);
+
+        # Remove `% ...` from the value of `Exec`
         index($info{Exec}, ' %') != -1 and $info{Exec} =~ s/ +%.*//s;
 
+        # Terminalize
         if (    $self->{terminalize}
             and defined($info{Terminal})
             and exists($TRUE_VALUES{$info{Terminal}})) {
             $info{Exec} = sprintf($self->{terminalization_format}, $self->{terminal}, $info{Exec});
         }
 
+        # Check and clean the icon name
         if (exists $info{Icon}) {
             my $icon = $info{Icon};
 
+            my $abs;
             if (substr($icon, 0, 1) eq '/') {
-                $info{Icon} = (-f $icon) ? $icon : '';
+                if (-f $icon) {    # icon is specified as an absolute path
+                    $abs = 1;
+                }
+                else {             # ... otherwise, take its basename
+                    $icon = substr($icon, 1 + rindex($icon, '/'));
+                }
             }
-            elsif ((my $rindex = rindex($icon, '.')) != -1) {
-                $info{Icon} = substr($icon, 0, $rindex);
+
+            # Remove any icon extension
+            if (!$abs) {
+                $icon =~ s/\.(?:png|jpe?g|svg|xpm)\z//i;
             }
+
+            # Store the icon back into `%info`
+            $info{Icon} = $icon;
         }
 
+        # Push the entry into its belonging categories
         if (scalar(@categories)) {
             foreach my $category (@categories) {
                 push @{$file_data->{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
@@ -240,7 +264,7 @@ Key/value pair arguments may be provided to set up the initial state.
 
 By default,
 
-    Linux::DesktopFiles->new()
+    Linux::DesktopFiles->new();
 
 is equivalent with:
 
@@ -248,7 +272,7 @@ is equivalent with:
 
         terminal               => $ENV{TERM},
         terminalize            => 0,
-        terminalization_format => "%s -e '%s'"
+        terminalization_format => "%s -e '%s'",
 
         skip_entry       => [],
         skip_filename_re => [],
@@ -295,7 +319,7 @@ hash reference when calling C<$obj-E<gt>parse_desktop_files>.
 =item categories => [qw(Graphics Network AudioVideo ...)]
 
 Any valid categories from the desktop files. Any category not listed will be ignored
-or stored in the I<unknown_category_key> when C<keep_unknown_categories> is set to a true value.
+or stored in the B<unknown_category_key> when C<keep_unknown_categories> is set to a true value.
 
 =back
 
@@ -306,7 +330,7 @@ or stored in the I<unknown_category_key> when C<keep_unknown_categories> is set
 =item keep_unknown_categories => $bool
 
 When an item is not part of any specified category, will be stored inside the
-unknown category, specified by I<unknown_category_key>.
+unknown category, specified by B<unknown_category_key>.
 
 =item unknown_category_key => $name
 
@@ -320,7 +344,7 @@ any non-alpha numeric characters with an underscore. For example, "X-XFCE" becom
 
 =item terminal => $command
 
-This terminal command will be used when I<terminalize> is set to a true value.
+This terminal command will be used when B<terminalize> is set to a true value.
 
 =item terminalize => $bool
 
@@ -346,12 +370,12 @@ Used internally as:
 =item skip_filename_re => qr/regex/
 
 Skip any desktop file if its file name matches the regex.
-I<NOTE:> File names are from the last slash to the end.
+B<NOTE:> File names are from the last slash to the end.
 
 =item skip_entry  => [{key => 'KeyName', re => qr/REGEX/i}, {...}]
 
 Skip any desktop file if the value from a given key matches the specified regular expression.
-The I<key> can be any valid key from the desktop files.
+The B<key> can be any valid key from the desktop files.
 
 Example:
 
@@ -363,10 +387,10 @@ Example:
 =item substitutions => [{key => 'KeyName', re => qr/REGEX/i, value => 'Value'}, {...}]
 
 Substitute, by using a regex, in the values of the desktop files.
-The I<key> can be any valid key from the desktop files.
-The I<re> can be any valid regular expression. Anything matched by the regex, will be
-replaced the string stored in I<value>.
-For global matching/substitution, you need to set the I<global> key to a true value.
+The B<key> can be any valid key from the desktop files.
+The B<re> can be any valid regular expression. Anything matched by the regex, will be
+replaced the string stored in B<value>.
+For global matching/substitution, you need to set the B<global> key to a true value.
 
 Example:
 
@@ -393,7 +417,7 @@ Parse a list of desktop files into a HASH ref.
 =item $obj->parse_desktop_files()
 
 It returns a HASH reference which categories names as keys, and ARRAY references
-as values which contains HASH references with the keys specified in the I<keys_to_keep>
+as values which contains HASH references with the keys specified in the B<keys_to_keep>
 option, and values from the desktop files.
 
 The returned HASH reference might look something like this:
